/* gibbs.c - the Gibbs sampler for the Ising model
 *
 * Copyright (C) 1999, 2002  Jochen Voss.  */

static const  char  rcsid[] = "$Id: gibbs.c 5044 2003-07-09 08:02:23Z voss $";

#include <stdlib.h>
#include <math.h>

#include <X11/Xutil.h>

#include "libv/v.h"
#include "xising.h"


static int
rnd (int anz)
/* Returns a random integer `x' with `0 <= x < anz'.  */
{
#if HAVE_DRAND48
  return  (int)(anz*drand48());
#else
  return  (int)((double)anz*rand()/(RAND_MAX+1.0));
#endif
}

static double
urnd (void)
/* Return a random number in the range of `[0;1)'.  */
{
#if HAVE_DRAND48
  return  drand48 ();
#else
  return  rand()/(RAND_MAX+1.0);
#endif
}

static int
dh (const struct image *im, int i, int j, int c)
/* Return the change of total Energy, if a neighbour of (I,J) and would be
 * changed from color 1-C to C.  */
{
  return  (XGetPixel (im->data, i, j) == c) ? -4 : +4;
}

void
do_one_step (struct image *im)
{
  int  x, y;
  int  old_col, other_col;
  int  delta_h;
  double  e, q;

  x = rnd (im->width);
  y = rnd (im->height);

  old_col = XGetPixel (im->data, x, y);
  other_col = 1-old_col;

  delta_h = 0;
  if (x>0)  delta_h += dh (im, x-1, y, other_col);
  if (x<im->width-1)  delta_h += dh (im, x+1, y, other_col);
  if (y>0)  delta_h += dh (im, x, y-1, other_col);
  if (y<im->height-1)  delta_h += dh (im, x, y+1, other_col);

  e = exp (-beta*delta_h);
  q = e / (1+e);
  if (urnd () < q) {
    XPutPixel (im->data, x, y, other_col);
    im->energy += delta_h;
    if (other_col) {
      im->magn += 2;
    } else {
      im->magn -= 2;
    }
  }
}
