/* error.c - functions to signal errors
 *
 * Copyright (C) 1996, 97  Jochen Voss.  */

static const  char  rcsid[] = "$Id: error.c 5044 2003-07-09 08:02:23Z voss $";

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>

#include "v.h"


static void
emit_message (const char *msgstr, const char *format, va_list ap)
{
  fflush (NULL);
  if (interactive_flag) {
    if (msgstr)  fprintf (stderr, "%s: ", msgstr);
  } else {
    fprintf (stderr, "%s: ", myname);
  }
  if (! interactive_flag && isupper (format[0])) {
    /* We all hope, that '%' is no upper-case letter.  */
    fputc (tolower (format[0]), stderr);
    vfprintf (stderr, format+1, ap);
  } else {
    vfprintf (stderr, format, ap);
  }
  fputc ('\n', stderr);
}

void
fatal (const char *format, ...)
/* Signal a fatal error and quit immediately.  Try to conform
 * with the "GNU coding standards".  */
{
  va_list  ap;

  va_start (ap, format);
  emit_message ("Fatal error", format, ap);
  va_end (ap);
  abort ();
}

void
error (const char *format, ...)
/* Emit an error message and continue without further consequence.
 * Try to conform with the "GNU coding standards".  */
{
  va_list  ap;

  va_start (ap, format);
  emit_message ("Error", format, ap);
  va_end (ap);
}

void
warning (const char *format, ...)
/* Emit a warning and continue without further consequence.  Try to
 * conform with the "GNU coding standards".  */
{
  va_list  ap;

  va_start (ap, format);
  emit_message ("Warning", format, ap);
  va_end (ap);
}

void
message (const char *format, ...)
/* Emit an informational message if verbose mode is enabled and
 * continue without further consequence.  */
{
  va_list  ap;

  if (! verbose_flag)  return;
  va_start (ap, format);
  emit_message (NULL, format, ap);
  va_end (ap);
}
