/* v.h - global libv declarations
 *
 * Copyright (C) 1998  Jochen Voss.
 *
 * $Id: v.h 5044 2003-07-09 08:02:23Z voss $ */

#ifndef FILE_V_H_SEEN
#define FILE_V_H_SEEN

#include <stdlib.h>

typedef enum { FALSE, TRUE } bool;

#ifdef __GNUC__
#define  LIBV_CONST_FN  __attribute__ ((const))
#define  LIBV_NORETURN_FN  __attribute__ ((noreturn))
#define  LIBV_PRINTF_FN  __attribute__ ((format (printf, 1, 2)))
#else
#define  LIBV_CONST_FN
#define  LIBV_NORETURN_FN
#define  LIBV_PRINTF_FN
#endif

/* from "error.c" */
extern  void  fatal (const char *format, ...) 
					LIBV_NORETURN_FN LIBV_PRINTF_FN;
extern  void  error (const char *format, ...) LIBV_PRINTF_FN;
extern  void  warning (const char *format, ...) LIBV_PRINTF_FN;
extern  void  message (const char *format, ...) LIBV_PRINTF_FN;

/* from "xmalloc.c" */

extern  void *xmalloc (size_t);
extern  void *xrealloc (void *, size_t);
extern  void  xfree (void *);

/* from "vgetopt.c" */

struct voption {
  const char  *name;	/* long option name */
  char  short_name;		/* short option name */
  bool *flag_ptr;		/* where is the flag */
  bool  has_arg;		/* Is there an argument? */
  const char *arg_name;	/* name of the argument */
  const char *help;	/* "--help" line */
};

#define V_MIXED		1
#define V_ONE_HYPHEN	2
#define V_KEEP_UNKNOWN	4
#define V_NO_JOIN	8

extern  int  vgetopt (const struct voption *, const char **, int);
extern  void  vendopt (void);
extern  void  show_options (const struct voption *);

/* from "vinit.c" */

extern  char **orig_argv;
extern  int  orig_argc;

extern  char **xargv;
extern  int  xargc, xargv_len;

extern  char *myname;

extern  bool  interactive_flag;
extern  bool  verbose_flag;

extern  void  vinit (int argc, char **argv);
extern  void  vexit (void);


/** from "util.c" **/

extern  const char *home_directory (void);

extern  double  pt2inch (double x)  LIBV_CONST_FN;
extern  double  mm2inch (double x)  LIBV_CONST_FN;
#define  cm2inch(x)  ((x)/2.54)
extern  double  inch2cm (double x)  LIBV_CONST_FN;
extern  double  inch2pt (double x)  LIBV_CONST_FN;

extern  double  string2dimen (const char *)  LIBV_CONST_FN;
extern  int  vplot_sgn (double)  LIBV_CONST_FN;
extern  int  quantify (double, int)  LIBV_CONST_FN;

#endif /* FILE_V_H_SEEN */
