/* main.c - main file for xising
 *
 * Copyright (C) 1999, 2002  Jochen Voss.  */

static const  char  rcsid[] = "$Id: main.c 5852 2004-07-13 20:02:05Z voss $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <signal.h>
#include <time.h>
#include <sys/types.h>
#include <assert.h>
#include <errno.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xresource.h>
#if HAVE_LIBXPM
#include <X11/xpm.h>
#endif

#include "libv/v.h"
#include "xising.h"

#include "icon.xbm"


double  beta = 1;		/* inverse temperature */


#define r(x) #x
#define q(x) r(x)


#define optionTableCount 5
static XrmOptionDescRec optionTable[optionTableCount] = {
  {"-display", ".display", XrmoptionSepArg, (XPointer) NULL},
  {"-geometry", ".geometry", XrmoptionSepArg, (XPointer) NULL},
  {"-font", ".font", XrmoptionSepArg, (XPointer) NULL},
  {"-fn", ".font", XrmoptionSepArg, (XPointer) NULL},
  {"-synchronous", ".synchronous", XrmoptionNoArg, "on"},
};

static  const char *res_name;
static  char  res_class [] = "XIsing";
static  XrmDatabase  resourceDB;
static  Atom  wm_protocols, wm_delete_window;

Display *display;
int  screen_num;
static  int  x_res, y_res;

GC  gc;
Window  win;

static 	XFontStruct  *font_info;

/************************************************************
 * Auxiliary functions
 */

static const char *
get_home_dir (void)
{
  uid_t  my_uid = getuid ();
  struct passwd *pw_entry = getpwuid (my_uid);
  if (pw_entry == NULL) {
    fatal ("You don't exist (wrong uid ?)");
  }
  return  pw_entry->pw_dir;
}

/************************************************************
 * Resource management
 */

static void
read_resources (void)
{
  XrmDatabase  argDB;

  argDB = resourceDB;
  if (XResourceManagerString (display) != NULL) {
    resourceDB = XrmGetStringDatabase(XResourceManagerString(display));
  } else {
    const char *home = get_home_dir ();
    char *buffer;

    buffer = xmalloc (strlen(home) + strlen("/.Xdefaults") + 1);
    strcpy (buffer, home);
    strcat (buffer, "/.Xdefaults");
    resourceDB = XrmGetFileDatabase (buffer);
    xfree (buffer);
  }

  /* command line takes precedence over everything */
  XrmMergeDatabases (argDB, &resourceDB);
}

static const char *
query_resource (const char *name, const char *class)
/* Return a value from the resource database.
 * If no value ist found, the function returns NULL.  */
{
  XrmValue  res_val;
  char *res_type;
  char *namebuf;
  char *classbuf;
  const char *result;

  namebuf = xmalloc(strlen(res_name)+1+strlen(name)+1);
  sprintf (namebuf, "%s.%s", res_name, name);
  classbuf = xmalloc(strlen(res_class)+1+strlen(class)+1);
  sprintf (classbuf, "%s.%s", res_class, class);
  if (XrmGetResource (resourceDB, namebuf, classbuf,
		      &res_type, &res_val) == True) {
    assert (strcmp (res_type, "String") == 0);
    result = res_val.addr;
  } else {
    result = NULL;
  }
  xfree (namebuf);
  xfree (classbuf);

  return  result;
}

static void
open_display (void)
{
  const char *resource_val;

  resource_val = query_resource("display", "Display");
  if ((display = XOpenDisplay (resource_val)) == NULL) {
    fatal ("Cannot connect to the X server \"%s\"",
	   XDisplayName (resource_val));
  }
  screen_num = DefaultScreen (display);

  x_res = DisplayWidth(display, screen_num)*25.4/
    DisplayWidthMM(display, screen_num)+0.5;
  y_res = DisplayHeight(display, screen_num)*25.4/
    DisplayHeightMM(display, screen_num)+0.5;
}

static void
open_window (void)
{
  int  x, y, width, height, ret;
  unsigned  bwidth = 1;
  XSizeHints  *size_hints;
  XWMHints  *wm_hints;
  XClassHint  class_hints;
  char window_name[80];
  char *window_name_p;
  const char *icon_name = res_name;
  const char *user_geom;
  XTextProperty  windowName, iconName;
  Pixmap  icon_pixmap;

  /* Calculate the window's geometry */
  if ((size_hints = XAllocSizeHints()) == 0) {
    fatal ("Structure allocation for `size_hints' failed");
  }
  size_hints->flags = PMinSize | PBaseSize;
  size_hints->min_width = 80;
  size_hints->min_height = 80;
  size_hints->base_width = 0;
  size_hints->base_height = 0;
  user_geom = query_resource ("geometry", "Geometry");
  ret = XWMGeometry (display, screen_num, user_geom,
		     q(BOARD_WIDTH) "x" q(BOARD_HEIGHT),
		     bwidth, size_hints,
		     &x, &y, &width, &height, &(size_hints->win_gravity));
  size_hints->flags |= PWinGravity;
  if (ret&(XValue|YValue)) {
    size_hints->flags |= USPosition;
  } else if (x!=0 || y!=0) {
    size_hints->flags |= PPosition;
  }
  size_hints->flags |= (ret&(WidthValue|HeightValue)) ? USSize : PSize;
  
  win = XCreateSimpleWindow (display, RootWindow (display, screen_num),
			     x, y, width, height, bwidth,
			     BlackPixel (display, screen_num),
			     WhitePixel (display, screen_num));

  /* Create pixmap of depth 1 (bitmap) for icon */
  icon_pixmap = XCreateBitmapFromData (display, win, icon_bits,
				       icon_width, icon_height);
  
  /* Calculate the window's state hints */
  if ((wm_hints = XAllocWMHints ()) == NULL) {
    fatal ("Structure allocation for `wm_hints' failed");
  }
  wm_hints->flags = StateHint | InputHint | IconPixmapHint;
  wm_hints->initial_state = NormalState;
  wm_hints->input = True;
  wm_hints->icon_pixmap = icon_pixmap;
  class_hints.res_name = res_name;
  class_hints.res_class = res_class;

  sprintf (window_name, "XIsing (=%g)", beta);
  window_name_p = window_name;
  if (XStringListToTextProperty (&window_name_p, 1, &windowName) == 0) {
    fatal ("Structure allocation for windowName failed.");
  }
  if (XStringListToTextProperty (&icon_name, 1, &iconName) == 0) {
    fatal ("Structure allocation for iconName failed.");
  }
  XSetWMProperties (display, win,
		    &windowName, &iconName, xargv, xargc,
		    size_hints, wm_hints, &class_hints);
  
  XFree (iconName.value);
  XFree (windowName.value);
  XFree (wm_hints);
  XFree (size_hints);

  /* Select event types wanted */
  XSelectInput (display, win,
		ExposureMask | ButtonPressMask
		| KeyPressMask | StructureNotifyMask);

  /* Display window */
  XMapWindow (display, win);
}

static volatile  sig_atomic_t  expired = 0;

static void
init_timer (void)
{
  struct itimerval old, new;

  new.it_interval.tv_usec = 0;
  new.it_interval.tv_sec = 1;
  new.it_value.tv_usec = 0;
  new.it_value.tv_sec = 1;
  if (setitimer (ITIMER_REAL, &new, &old) < 0)
    fatal ("cannot set timer");
}

/* The signal handler just clears the flag and re-enables itself. */
RETSIGTYPE
catch_alarm (int sig)
{
  expired = 1;
  signal (sig, catch_alarm);
}

static int
input_ready (int fd)
{
  fd_set  rfds;
  int  retval;

  FD_ZERO (&rfds);
  FD_SET (fd, &rfds);

  do {
    struct timeval  tv;
    
    tv.tv_sec = 0;
    tv.tv_usec = 0;
    retval = select (FD_SETSIZE, &rfds, NULL, NULL, &tv);
  } while (retval<0 && errno == EINTR);
  if (retval < 0) {
    fatal ("Select failed: %s", strerror (errno));
  }
  do {
    struct timeval  tv;
    
    tv.tv_sec = 0;
    tv.tv_usec = 0;
    retval = select (FD_SETSIZE, &rfds, NULL, NULL, &tv);
  } while (retval<0 && errno == EINTR);
  if (retval < 0) {
    fatal ("Select failed: %s", strerror (errno));
  }

  return  retval;
}

static bool
handle_x_events (const struct image *im)
{
  XEvent report;
  bool  done = FALSE;

  while (input_ready (ConnectionNumber (display))) {
    XNextEvent (display, &report);
    switch (report.type) {
    case Expose:
      image_redraw (im, report.xexpose.x, report.xexpose.y,
		    report.xexpose.width, report.xexpose.height);
      break;
    case ClientMessage:
      if (report.xclient.message_type == wm_protocols
	  && report.xclient.format == 32
	  && report.xclient.data.l[0] == wm_delete_window) {
	done = TRUE;
      }
      break;
#if 0
    case KeyPress:
      done = TRUE;
      break;
#endif
    default:
      break;
    }
  }
  
  return  done;
}

int
main (int argc, char **argv)
{
  struct image *im;
  bool  error_flag = FALSE;	/* Show usage and exit.  */
  bool  help_flag = FALSE;	/* Show help and exit (-h).  */
  bool  omit_data_flag = FALSE; /* no data on stdout (-q) */
  bool  omit_window_flag = FALSE; /* no window (-o) */
  struct voption  option_tab [] = {
    { "beta", 'b', NULL, TRUE, "B", "set the inverse temperature to B" },
#if HAVE_LIBXPM
    { "dump", 'd', NULL, TRUE, "FILE",
      "dump the final image to FILE (format: xpm)" },
#endif
    { "help", 'h', &help_flag, FALSE, NULL, "show this message and exit" },
    { "name", 'N', NULL, TRUE, "RES_NAME",
      "set the application resource name to `RES_NAME'" },
    { "omit-data", 'q', &omit_data_flag, FALSE, NULL,
      "Do not emit energy and magnetisation to stdout" },
    { "omit-window", 'o', &omit_window_flag, FALSE, NULL,
      "Do not open any window" },
    { "steps", 'n', NULL, TRUE, "COUNT",
      "stop after COUNT steps" },
    { NULL, '\0', NULL, FALSE, NULL, NULL }
  };
  char  buffer [128];
  const char *resource_val;
#if HAVE_LIBXPM
  const char *dump_file_name = NULL;
#endif
  long int  iteration, limit;
  bool  done;

#if HAVE_DRAND48
  srand48 (time (NULL));
#else
  srand (time (NULL));
#endif

  vinit (argc, argv);
  res_name = myname;

  limit = -1;
  while (! error_flag) {
    const char *argptr;
    int  c = vgetopt (option_tab, &argptr,
		      V_ONE_HYPHEN|V_KEEP_UNKNOWN|V_MIXED);

    if (c == -1)  break;
    switch (c) {
    case 'b':
      {
	char *tail;
	double  val;
	
	errno = 0;
	val = strtod (argptr, &tail);
	if (errno == ERANGE && *tail == '\0') {
	  fprintf (stderr, "Die Zahl \"%s\" ist unzulssig!\n", argptr);
	  error_flag = TRUE;
	} else if (errno || tail == argptr || *tail != '\0') {
	  fprintf (stderr, "Der Ausdruck \"%s\" ist keine ordentliche Zahl!\n",
		   argptr);
	  error_flag = TRUE;
	} else {
	  beta = val;
	}
      }
      break;
#if HAVE_LIBXPM
    case 'd':
      dump_file_name = argptr;
      break;
#endif
    case 'n':
      {
	char *tail;
	long int  val;
	
	errno = 0;
	val = strtol (argptr, &tail, 10);
	if (errno == ERANGE && *tail == '\0') {
	  fprintf (stderr, "Die Zahl \"%s\" ist unzulssig!\n", argptr);
	  error_flag = TRUE;
	} else if (errno || tail == argptr || *tail != '\0') {
	  fprintf (stderr, "Der Ausdruck \"%s\" ist keine ordentliche Zahl!\n",
		   argptr);
	  error_flag = TRUE;
	} else {
	  limit = val;
	}
      }
      break;
    case 'N':
      res_name = argptr;
      break;
    case 0:
      error_flag = TRUE;
      break;
    default:
      assert (0);
    }
  }
  /* Give an initial value to the `resourceDB' (to see which display
   * to open).  This value is later-on modified by the function
   * `read_resources'.  */
  XrmInitialize ();
  XrmParseCommand (&resourceDB, optionTable, optionTableCount,
		   res_name, &xargc, xargv);
  if (!error_flag && xargc > 1) {
    int  i;

    for (i=1; i<xargc && xargv[i][0]=='-'; ++i) {
      error ("Unknown option \"%s\"", xargv[i]);
    }
    if (i==1 || i+1<xargc)  error ("Too much arguments");
    fputc ('\n', stderr);
    error_flag = TRUE;
  }
  if (error_flag || help_flag) {
    fprintf (stderr, "usage: %s [options]\n", myname);
    if (error_flag) {
      fprintf (stderr, "Try `%s --help' for more information.\n", myname);
      if (! help_flag)  exit (1);
    }
    fputc ('\n', stderr);
    show_options (option_tab);
    puts ("Long options may start with one or two hyphens\n");
    fputc ('\n', stderr);
    puts ("There are additional options, specific to the X Window System.\n");
    exit (error_flag ? EXIT_FAILURE : EXIT_SUCCESS);
  }

  open_display ();
  
  wm_protocols = XInternAtom (display, "WM_PROTOCOLS", False);
  wm_delete_window = XInternAtom (display, "WM_DELETE_WINDOW", False);
  
  read_resources ();

  resource_val = query_resource ("synchronous", "Synchronous");
  if (resource_val && strcmp (resource_val, "on") == 0) {
    warning ("Using synchrone mode");
    XSynchronize (display, True);
  }

  /* Load font and get font information structure. */
  resource_val = query_resource ("font", "Font");
  if (resource_val == NULL) {
    sprintf (buffer, "-*-courier-*-r-*-*-*-120-%d-%d-m-*-iso8859-1",
	     x_res, y_res);
    resource_val = buffer;
  }
  if ((font_info = XLoadQueryFont (display, resource_val)) == NULL) {
    fatal ("Cannot open font \"%s\"", resource_val);
  }

  if (! omit_window_flag) {
    open_window ();
    XSetWMProtocols (display, win, &wm_delete_window, 1);

    /* create GC for text and drawing */
    gc = XCreateGC (display, win, 0, NULL);
    XSetFont (display, gc, font_info->fid);
    XSetForeground (display, gc, BlackPixel (display, screen_num));
    XSetBackground (display, gc, WhitePixel (display, screen_num));
    XSetLineAttributes (display, gc, 1, LineSolid, CapProjecting, JoinMiter);
  }

  im = new_image (win, BOARD_WIDTH, BOARD_HEIGHT);
  image_fill_random (im);

  signal (SIGALRM, catch_alarm);
  if (! omit_window_flag)  init_timer ();
  
  done = handle_x_events (im);
  iteration = 0;
  do {
    if (! omit_data_flag && iteration % 1000 == 0) {
      printf ("%6ld %d %d\n", iteration/1000, im->energy, im->magn);
    }
    do_one_step (im);
    ++iteration;
    if (expired) {
      image_redraw (im, 0, 0, im->width, im->height);
      done = handle_x_events (im);
      expired = FALSE;
    }
  } while (! done && (limit < 0 || iteration <= limit));

#if HAVE_LIBXPM
  if (dump_file_name)
    XpmWriteFileFromImage (display, dump_file_name, im->data, NULL, NULL);
#endif

  if (! omit_window_flag) {
    XFreeFont (display, font_info);
    XFreeGC (display, gc);
  }
  delete_image (im);
  XCloseDisplay (display);

  vexit ();

  return  0;
}
